#include "definition_finder.h"

#include "../internal_representation/design.h"

SgFunctionDefinition*
risc::traverser::get_function_definition(SgFunctionDeclaration *func_decl)
{
  SgProject *project = risc::get_project();

  // Loop over the files in the project (multiple files exist
  // when multiple source files are placed on the command line).
  for(SgFilePtrList::const_iterator
      file  = project->get_fileList().begin();
      file != project->get_fileList().end();
      ++file) {

    SgSourceFile *source_file = isSgSourceFile(*file);

    Rose_STL_Container<SgNode*> temp_decls
      = NodeQuery::querySubTree(source_file->get_globalScope(),
                       V_SgFunctionDefinition);

    for(Rose_STL_Container<SgNode*>::const_iterator
        temp_iter  = temp_decls.begin();
        temp_iter != temp_decls.end();
        ++temp_iter) {

      SgFunctionDefinition *temp_def = isSgFunctionDefinition(*temp_iter);

      if(SageInterface::isSameFunction(func_decl,temp_def->get_declaration())) {

        return temp_def;
      }
    }
  }

  std::cout << "Caution: RETURNING NULL POINTER, SEE BELOW" << std::endl;
  std::cout << SageInterface::get_name(func_decl) << std::endl;
  return NULL;
}

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
