#ifndef OOO_INSTRUMENTATION_H_INCLUDED_
#define OOO_INSTRUMENTATION_H_INCLUDED_

#include <iostream>
#include <cstdlib>

#include "rose.h"

#include "../internal_representation/design.h"
#include "../segment_graph/data_conflict_table.h"
#include "../segment_graph/event_conflict_table.h"
#include "../segment_graph/time_advanced_table.h"
#include "../tools/class_is_derived_from.h"

namespace risc {

namespace inst {

class InstrumentationTraversal: public AstSimpleProcessing
{
  public:
    InstrumentationTraversal(SgType*, SgExpression*, SgExpression*, 
                             SgClassDefinition*, std::map<SgNode*, int>&);
    virtual void visit(SgNode* astNode);

  private:
    SgType* variableType;
    SgExpression* PrimChnlVarRefExp;
    SgExpression* SCChnlVarRefExp;
    SgClassDefinition* SCModuleClassDef;
    std::map<SgNode*, int>& FuncCallExptoID;
};

// Tailored backend for SystemC source files
// Modified from ~src/roseSupport/utility_functions.C
int 
risc_backend (SgProject* project, UnparseFormatHelp *unparseFormatHelp = NULL,
              UnparseDelegate* unparseDelagate = NULL,
      unsigned int VerbosityLevel = 0); //....TODO risc::inst::ParseStatus status // hack by RD, 10/30/15, FIXME

int
instrumentor (risc::Design& design, risc::sg::DataConflictTable* data_conflict_table, 
  risc::sg::EventConflictTable* event_conflict_table,
  risc::sg::TimeAdvancedTable* time_advance_table,
  std::map<SgNode*, int>& FuncCallExptoID,
      unsigned int VerbosityLevel = 0); //....TODO risc::inst::ParseStatus status // hack by RD, 10/30/15, FIXME

struct ParseStatus
{
  unsigned int VerbosityLevel;
  unsigned int WarningLevel;

  ParseStatus();
  ParseStatus& operator=(const ParseStatus& copy);
};

ParseStatus
ParseCommandlineOptions(int &argc, char** &argv);


} // end of namespace inst

} // end of namespace risc

#endif /* OOO_INSTRUMENTATION_H_INCLUDED_ */

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
