#ifndef CTHREAD_H_INCLUDED_
#define CTHREAD_H_INCLUDED_

#include <vector>
#include <utility>

#include "rose.h"
#include "function.h"
#include "port.h"
#include "inport.h"
#include "channel.h"

namespace risc {

// These are the reset signal types. A reset signal
// can be represented by a in/inout port
typedef std::pair<Port*, SgBoolValExp*> ResetSignal;
//typedef std::pair<Port*, SgBoolValExp*> AsyncResetSignal;

typedef std::vector<ResetSignal*> ResetSignalVector;
typedef std::vector<ResetSignal*>::iterator ResetSignalVectorIter;
typedef std::vector<ResetSignal*>::const_iterator ResetSignalVectorConstIter;

//typedef std::vector<AsyncResetSignal*>
//            AsyncResetSignalVector;
//typedef std::vector<AsyncResetSignal*>::iterator
//            AsyncResetSignalVectorIter;
//typedef std::vector<AsyncResetSignal*>::const_iterator
//            AsyncResetSignalVectorConstIter;

class CThread: public Function {

public:

  /**
   * \brief Default constructor
   */
  explicit CThread(SgFunctionDefinition *ast_node);


  InPort *clk_port_;

  ResetSignalVector reset_signal_;
  ResetSignalVector async_reset_signal_;

private:
  /**
   * \brief Disabled copy constructor
   */
  CThread(const CThread &t);
};

typedef std::vector<CThread*> CThreadVector;
typedef std::vector<CThread*>::iterator CThreadVectorIter;
typedef std::vector<CThread*>::const_iterator CThreadVectorConstIter;

}; // end namespace risc

#endif /* CTHREAD_H_INCLUDED_ */

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
