#ifndef SYSTEMC_DEFINITION_FINDER_H_INCLUDED_
#define SYSTEMC_DEFINITION_FINDER_H_INCLUDED_

#include "rose.h"

namespace risc {

namespace tools {

/**
 * \brief This class stores all SystemC definition pointer
 *
 */
class SystemCDefinitionFinder {

public:
  static SystemCDefinitionFinder* get_definitions(SgProject *project);

  /**
   * \brief Pointer to the sc_module definition in the rose ast tree
   */
  SgClassDefinition *sc_module_;
  SgClassDefinition *sc_port_;
  SgClassDefinition *sc_port_base_;
  SgClassDefinition *sc_in_;
  SgClassDefinition *sc_out_;
  SgClassDefinition *sc_inout_;
  SgClassDefinition *sc_event_;
  SgClassDefinition *sc_prim_channel_;
  SgClassDefinition *sc_interface_;

  // This is a hack. Please see the comment in the source file.
  SgClassDefinition *sc_channel_;
  SgClassDefinition *sc_in_clk_;
  SgClassDefinition *sc_out_clk_;
  SgClassDefinition *sc_inout_clk_;

  // Namespace of systemc
  SgNamespaceDefinitionStatement *sc_core_;
  SgNamespaceDefinitionStatement *sc_dt_;

  std::set<SgFunctionDefinition*> sc_wait_funcs_;

private:
  SgProject *project_;

  void init_systemc_definitions();

  static SystemCDefinitionFinder *instance;
  SystemCDefinitionFinder(SgProject *project);
  ~SystemCDefinitionFinder();
  SystemCDefinitionFinder(const SystemCDefinitionFinder &other);
};


}; // end of namespace tools

}; // end of namespace risc

#endif /* SYSTEMC_DEFINITION_FINDER_H_INCLUDED_ */

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
