#include <iostream>
#include <set>
#include <string>
#include <algorithm>

#include "rose.h"

#include "risc.h"

int main(int argc, char* argv[])
{
  std::cout << std::endl << "START PROCESSING FILE: " __FILE__ << " ..." << std::endl;

  risc::Design d(argc,argv);

  AstTests::runAllTests(d.project_);

  std::vector<SgClassDefinition*> modules = risc::traverser::find_all_modules();

  std::set<std::string> expected_output;
  expected_output.insert("DerivedSCmoduleA");
  expected_output.insert("DerivedSCmoduleB");
  expected_output.insert("DerivedSCmoduleC");
  expected_output.insert("M");

  std::set<std::string> output;

  for(std::vector<SgClassDefinition*>::iterator iter = modules.begin();
    iter != modules.end(); iter++) {
    std::cout << get_name(*iter) << std::endl;

    output.insert(get_name(*iter));
  }

  if(expected_output.size() != output.size()) {
    std::cout << "expected output and output have different sizes" << std::endl;
    std::cout << "expected_output.size(): " << expected_output.size() << std::endl;
    std::cout << "output.size(): " << output.size() << std::endl;
    assert(false);
  }

  if(!std::equal(expected_output.begin(), expected_output.end(), output.begin() )) {
    std::cout << "expected output and output have different elements" << std::endl;
    assert(false);
  }

  std::cout << "================== (unparse end) ==================" << std::endl;

  std::cout << "END PROCESSING FILE: " __FILE__ << " ..." << std::endl << std::endl;

  return backend(d.project_);
}

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
