#include "systemc.h"

// +----------------------------------------+
// |                   TOP                  |
// |+-------------------++-------++--------+|
// ||        A1         ||  A2   ||   C    ||
// ||+-------++--------+||       ||        ||
// |||  M    ||   M    |||       ||        ||
// |||       ||        |||       ||        ||
// |||       ||        |||       ||        ||
// |||       ||        |||       ||        ||
// |||       ||        |||       ||        ||
// ||+-------++--------+||       ||        ||
// |+-------------------++-------++--------+|
// +----------------------------------------+

class InstanceEventConflictTableM: public sc_module {
public:

  SC_HAS_PROCESS(InstanceEventConflictTableM);
  sc_event &event_both_;
  sc_event &event_individual_;

  InstanceEventConflictTableM(
      sc_module_name mn,
      sc_event &event_both,
      sc_event &event_individual):
    sc_module(mn),
    event_both_(event_both),
    event_individual_(event_individual)
  {
    SC_THREAD(thread1);
  }

  void thread1()
  {
    wait(SC_ZERO_TIME);

    wait(event_both_);

    wait(event_individual_);
  }
};

class InstanceEventConflictTableA1: public sc_module {
public:

  SC_HAS_PROCESS(InstanceEventConflictTableA1);

  InstanceEventConflictTableM m1;
  InstanceEventConflictTableM m2;
  sc_event event_a_1;
  sc_event event_a_1_to_m1;
  sc_event event_a_1_to_m2;

  InstanceEventConflictTableA1(sc_module_name mn):
    sc_module(mn),
    m1("m1", event_a_1, event_a_1_to_m1),
    m2("m2", event_a_1, event_a_1_to_m2)
  {
    SC_THREAD(thread1);
  }

  void thread1()
  {
    event_a_1.notify(SC_ZERO_TIME);
    event_a_1_to_m1.notify(SC_ZERO_TIME);
    //event_a_1_to_m2.notify(SC_ZERO_TIME);
  }
};

class InstanceEventConflictTableA2: public sc_module {
public:

  SC_HAS_PROCESS(InstanceEventConflictTableA2);

  InstanceEventConflictTableA2(sc_module_name mn):
    sc_module(mn)
  {
    SC_THREAD(thread1);
  }

  void thread1()
  {
    wait(SC_ZERO_TIME);

    wait(SC_ZERO_TIME);
  }
};


class InstanceEventConflictTableC: public sc_module {
public:

  SC_HAS_PROCESS(InstanceEventConflictTableC);

  sc_event both;
  sc_event to_thread_4;
  sc_event to_thread_5;

  InstanceEventConflictTableC(sc_module_name mn):
    sc_module(mn)
  {
    //SC_THREAD(thread1);
    //SC_THREAD(thread2);
    SC_THREAD(thread3);
    SC_THREAD(thread4);
    SC_THREAD(thread5);
  }

  void thread1()
  {
    func();
  }

  void thread2()
  {
    func();
  }

  void func()
  {
    wait(SC_ZERO_TIME);

    wait(SC_ZERO_TIME);

    wait(SC_ZERO_TIME);

    wait();
  }

  void thread3()
  {
    both.notify(SC_ZERO_TIME);

    wait(SC_ZERO_TIME);

    to_thread_4.notify(SC_ZERO_TIME);
    to_thread_5.notify(SC_ZERO_TIME);

    wait(SC_ZERO_TIME);
    to_thread_4.notify(SC_ZERO_TIME);

    wait(SC_ZERO_TIME);
    to_thread_5.notify(SC_ZERO_TIME);
  }

  void thread4()
  {
    wait(both);

    wait(to_thread_4);

    wait(to_thread_4);
  }

  void thread5()
  {
    wait(both);

    wait(to_thread_5);

    wait(to_thread_5);
  }
};

class SomeClassA: public sc_module {

public:
  SC_HAS_PROCESS(SomeClassA);
  sc_event &event_a;

  SomeClassA(sc_module_name mn, sc_event &param_event_a):
      sc_module(mn),
      event_a(param_event_a) {

    SC_THREAD(thread1);
  }

  void thread1()
  {
    event_a.notify(SC_ZERO_TIME);
  }
};

class SomeClassB: public sc_module {

public:
  SC_HAS_PROCESS(SomeClassB);
  sc_event &event_b;

  SomeClassB(sc_module_name mn, sc_event &param_event_b):
      sc_module(mn),
      event_b(param_event_b) {

    SC_THREAD(thread1);
  }

  void thread1()
  {

    wait(event_b);
  }
};

class EventConflictTable: public sc_module {
public:

  SC_HAS_PROCESS(EventConflictTable);

  sc_event event_1;
  sc_event event_2;
  sc_event event_3;

  sc_event some_event;

  SomeClassA some_class_a;
  SomeClassB some_class_b;

  void thread1()
  {
    event_1.notify(SC_ZERO_TIME);

    wait(SC_ZERO_TIME);

    wait(event_2);

    wait(SC_ZERO_TIME);
  }

  void thread2()
  {
    wait(event_1);

    event_2.notify(SC_ZERO_TIME);
    wait(SC_ZERO_TIME);
    event_3.notify(SC_ZERO_TIME);
  }

  EventConflictTable(sc_module_name mn):
      sc_module(mn),
      some_class_a("some_class_a", some_event),
      some_class_b("some_class_b", some_event)
  {
    SC_THREAD(thread1);
    SC_THREAD(thread2);
  }
};

class InstanceEventConflictTableTop: public sc_module {
public:

  InstanceEventConflictTableA1 a1;
  InstanceEventConflictTableA2 a2;
  InstanceEventConflictTableC c;
  EventConflictTable ect;

  InstanceEventConflictTableTop(sc_module_name mn):
    sc_module(mn),
    a1("a1"),
    a2("a2"),
    c("c"),
    ect("ect")
  {

  }
};

#ifndef RISC_ALL_SEGMENT_TESTS
int sc_main(int argc, char **argv)
{
  InstanceEventConflictTableTop mm("mm");

  sc_start();
  return 0;
}
#endif

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
